
import os, re
import flask, json
import subprocess
import requests
from flask import request, Response
from flask_cors import *

NFS_SERVER = '154.92.111.106'
FILE_SERVER_URL = 'http://154.92.108.28/'
K8S_URL = 'http://192.168.0.107:8089/'
PLATFORM_PARAM = [
	{ "name" : "国际", "ns" : "intersms", "type" : "k8s", "ips" : ["8.210.1.22"] },
	{ "name" : "国际测试", "ns" : "testsms", "type" : "k8s", "ips" : ["8.210.1.22"] },
	{ "name" : "Sendmax", "ns" : "sendmax-sms", "type" : "k8s", "ips" : ["8.210.1.22"] },
	{ "name" : "灰鸽子", "ns" : "sms-newc", "type" : "k8s", "ips" : ["8.218.111.151"] },
	{ "name" : "云迅", "ns" : "sms-yx", "type" : "k8s", "ips" : ["8.218.111.151"] },
	{ "name" : "综合", "ns" : "sms-comp", "type" : "k8s", "ips" : ["8.218.111.151"] },
	{ "name" : "易云", "ns" : "sms-cmi-ec", "type" : "k8s", "ips" : ["8.218.111.151"] },
	{ "name" : "海豚", "ns" : "sms-cmi-ht", "type" : "k8s", "ips" : ["8.218.111.151"] },
	{ "name" : "全通", "ns" : "sms-cmi-qt", "type" : "k8s", "ips" : ["8.218.111.151"] },
	{ "name" : "创新", "ns" : "sms-cmi-ph", "type" : "k8s", "ips" : ["8.218.111.151"] },
]

# 请求接口
SERVER = flask.Flask(__name__)
CORS(SERVER, resources=r'/*')

# 获取所有平台信息
@SERVER.route('/getNameSpace', methods=['get', 'post'])
def getPlatforms():
	return responseData(PLATFORM_PARAM)


# ================================ k8s 请求 ================================ 
# 获取应用
def k8sGetPods(ns, ip):
	
	url = K8S_URL + 'getPods?ns=' + ns
	print('请求地址: ' + url)
	
	obj = requests.get(url)
	if obj.status_code != 200: 
		return responseData(None, "不存在当前平台")

	names = obj.json()['data']['list']
	resList = [{"ip" : ip, "pod" : name} for name in names]
	return responseData(resList)

# 开始抓包
def k8sPacketCapture(ns, pod, ipaddr):

	url = K8S_URL + 'packetCapture?ns=' + ns + '&pod=' + pod
	if bool(ipaddr) != 0: url += '&ipaddr=' + ipaddr		
	print('请求地址: ' + url)

	obj = requests.get(url)
	if obj.status_code != 200: 
		return responseData(None, "不存在当前平台")

	return responseData(obj.json()['data'])

# ================================ k8s 请求 ================================ 


# ================================ 通用请求 ================================ 
# 获取应用
@SERVER.route('/getPods', methods=['get', 'post'])
def getPods():

	ns = request.values.get('ns')
	if ns is None:
		return responseData(None, "应用别名不可为空")

	# 拿到字典
	param = {}
	for x in PLATFORM_PARAM:
		if ns in x.values():
			param = x

	# 判断平台是否存在
	if bool(param) == 0:
		return responseData(None, "不存在当前平台")

	# 判断是否K8S
	if param['type'] == 'k8s':
		return k8sGetPods(ns, param['ips'][0])
		
	# Docker 处理
	pods = []
	for ip in param['ips']:
		names = runCommand(ip,  'docker ps | grep sms | grep -v nginx', '| awk \'{print $NF}\'')
		names = list(map(lambda x: "-".join(re.findall(r"sms|gateway|sort|\w+", x)), names))
		resList = [{"ip" : ip, "pod" : name} for name in names]
		pods.extend(resList)

	return responseData(pods)


# 开始抓包
@SERVER.route('/packetCapture', methods=['get', 'post'])
def packetCapture():

	# 获取参数
	ns = request.values.get('ns')
	ip = request.values.get('ip')
	pod = request.values.get('pod')
	ipaddr = request.values.get('ipaddr')
	if bool(ns) == 0 or bool(ip) == 0 or bool(pod) == 0:
		return responseData(None, "参数不可为空")

	# 拿到字典
	param = {}
	for x in PLATFORM_PARAM:
		if ns in x.values():
			param = x

	# 判断是否K8S
	if param['type'] == 'k8s':
		return k8sPacketCapture(ns, pod, ipaddr)
	
	# 判断是否有NFS
	nfsStatus = runCommand(ip, f'mount | grep {NFS_SERVER}')
	# 不存在则安装/挂载
	if bool(nfsStatus) == 0: 
		runCommand(ip, 'yum -y install nfs-utils')
		path = runCommand(ip, f'showmount -e {NFS_SERVER} | grep pcap')[0]
		runCommand(ip, 'mkdir /pcap')
		runCommand(ip, f'mount -t nfs {NFS_SERVER}:{path} /pcap && chmod 777 /pcap')
	
	# 检测应用
	podName = pod.replace('-', '')
	containerId = runCommand(ip, f'docker ps | grep {podName}', '| awk \'{print $1}\'')
	if bool(containerId) == 0: return responseData(None, f"不存在该应用")
	
	# 开始抓包
	containerPid = runCommand(ip, f'docker inspect -f {{{{.State.Pid}}}} {containerId[0]}')[0]
	packetName = f'{ns}-{pod}-{ipaddr}.pcap'
	hostCmd = f'host {ipaddr}' if ipaddr else ''
	runCommand(ip, f'nsenter -t {containerPid} -n tcpdump -i eth0 {hostCmd} -w /pcap/{packetName}', isSystem = True)

	# 获取进程号
	pid = runCommand(ip, f'ps -ef | grep {packetName}')
	if bool(pid) != 0:
		pid = pid[1]

	# 开始抓包到NFS上，并返回PID，下载路径
	filePath = FILE_SERVER_URL + packetName
	data = { 'pid' : pid, 'filePath' : filePath }
	return responseData(data)


# 停止抓包
@SERVER.route('/stopCapture', methods=['get', 'post'])
def stopCapture():

	ip = request.values.get('ip')
	pid = request.values.get('pid')
	if not pid or not ip:
		return responseData(None, "PID与IP地址不为可为空")

	runCommand(ip, f'kill -9 {pid}')
	return responseData({})
# ================================ 通用请求 ================================ 


# 运行命令
def runCommand(ip, cmd, localCmd = '', isSystem = False):
	
	cmd = f'ssh -i ~/.ssh/hkqtkey.pem root@{ip} ' +  f'\"{cmd}\" ' + f'{localCmd}'
	print("运行命令: " + cmd)

	if isSystem:
		return subprocess.Popen(cmd, shell = True)

	return os.popen(cmd).read().split()

# 返回请求数据
def responseData(data, message = ''):
	
	res = ''
	if data is not None: res = {'code': 200, 'data': data, 'message': '请求成功'}
	else: res = {'code': 500, 'data': {}, 'message': message}
	return Response(json.dumps(res), mimetype='application/json')


if __name__ == '__main__':

	SERVER.run(debug=True, port=8088, host='0.0.0.0')
